@@ -38,7 +38,8 @@ def scan(
     in_axes: Any,
     out_axes: Any,
     length: Optional[int] = None,
-    reverse: bool = False):
+    reverse: bool = False,
+    unroll: int = 1):
   """A wrapper around `jax.lax.scan` with in_axes/out_axes api.
 
   Example::
@@ -70,6 +71,8 @@ def scan(
     length: number of iterations. Only needs to be specified if there
       is no scan axis from which it can be derived.
     reverse: scan in reverse order from end to start.
+    unroll: how many scan iterations to unroll within a single
+      iteration of a loop (default: 1).
   Returns:
      the function that performs the scan of the form:
      (broadcast_in, carry_in, *args) -> (broadcast_out, carry_out, scan_out).
@@ -84,7 +87,7 @@ def scan(
       perm = tuple(range(x.ndim))
       perm = (ax,) + tuple(np.delete(perm, ax))
       return jnp.transpose(x, perm)
-    return jax.tree_map(trans, xs)
+    return jax.tree_util.tree_map(trans, xs)
 
   def transpose_from_front(ax, xs):
     if ax is broadcast:
@@ -99,36 +102,36 @@ def scan(
       assert pax < x.ndim
       perm = tuple(range(1, pax + 1)) + (0,) + tuple(range(pax + 1, x.ndim))
       return jnp.transpose(x, perm)
-    return jax.tree_map(trans, xs)
+    return jax.tree_util.tree_map(trans, xs)
 
   def scan_fn(broadcast_in, init, *args):
-    xs = jax.tree_map(transpose_to_front, in_axes, args)
+    xs = jax.tree_util.tree_map(transpose_to_front, in_axes, args)
 
     def body_fn(c, xs, init_mode=False):
       # inject constants
-      xs = jax.tree_map(lambda ax, arg, x: (arg if ax is broadcast else x),
-                             in_axes, args, xs)
+      xs = jax.tree_util.tree_map(
+          lambda ax, arg, x: (arg if ax is broadcast else x), in_axes, args, xs)
       broadcast_out, c, ys = fn(broadcast_in, c, *xs)
 
       if init_mode:
-        ys = jax.tree_map(lambda ax, y: (y if ax is broadcast else ()),
-                               out_axes, ys)
+        ys = jax.tree_util.tree_map(
+            lambda ax, y: (y if ax is broadcast else ()), out_axes, ys)
         return broadcast_out, ys
       else:
-        ys = jax.tree_map(lambda ax, y: (() if ax is broadcast else y),
-                               out_axes, ys)
+        ys = jax.tree_util.tree_map(
+            lambda ax, y: (() if ax is broadcast else y), out_axes, ys)
         return c, ys
     broadcast_body = functools.partial(body_fn, init_mode=True)
 
-    carry_avals = jax.tree_map(
+    carry_avals = jax.tree_util.tree_map(
         lambda x: jax.ShapedArray(jnp.shape(x), jnp.result_type(x)),
         init)
-    scan_avals = jax.tree_map(
+    scan_avals = jax.tree_util.tree_map(
         lambda x: jax.ShapedArray(jnp.shape(x)[1:], jnp.result_type(x)),
         xs)
     input_avals = (carry_avals, scan_avals)
 
-    in_avals, in_tree = jax.tree_flatten(input_avals)
+    in_avals, in_tree = jax.tree_util.tree_flatten(input_avals)
     f_flat, out_tree = jax.api_util.flatten_fun_nokwargs(
         lu.wrap_init(broadcast_body), in_tree)
     in_pvals = list(map(pe.PartialVal.unknown, in_avals))
@@ -140,11 +143,12 @@ def scan(
         raise ValueError(
             'broadcasted variable has a data dependency on the scan body.')
       out_flat.append(const)
-    broadcast_in, constants_out = jax.tree_unflatten(out_tree(), out_flat)
+    broadcast_in, constants_out = jax.tree_util.tree_unflatten(out_tree(), out_flat)
 
-    c, ys = lax.scan(body_fn, init, xs, length=length, reverse=reverse)
-    ys = jax.tree_map(transpose_from_front, out_axes, ys)
-    ys = jax.tree_map(
+    c, ys = lax.scan(body_fn, init, xs, length=length,
+                     reverse=reverse, unroll=unroll)
+    ys = jax.tree_util.tree_map(transpose_from_front, out_axes, ys)
+    ys = jax.tree_util.tree_map(
         lambda ax, const, y: (const if ax is broadcast else y), out_axes,
         constants_out, ys)
     return broadcast_in, c, ys
