@@ -15,7 +15,7 @@
 """Frozen Dictionary."""
 
 import collections
-from typing import Any, TypeVar, Mapping, Dict, Tuple
+from typing import Any, TypeVar, Mapping, Dict, Tuple, Union, Hashable
 
 from flax import serialization
 import jax
@@ -47,6 +47,7 @@ def _indent(x, num_spaces):
   return '\n'.join(indent_str + line for line in lines[:-1]) + '\n'
 
 
+# TODO(ivyzheng): change to register_pytree_with_keys_class after JAX release.
 @jax.tree_util.register_pytree_node_class
 class FrozenDict(Mapping[K, V]):
   """An immutable variant of the Python dict."""
@@ -148,19 +149,26 @@ class FrozenDict(Mapping[K, V]):
     """
     return unfreeze(self)
 
-  def tree_flatten(self) -> Tuple[Tuple[Dict[Any, Any]], Tuple[()]]:
+  # TODO(ivyzheng): remove this after JAX 0.4.6 release.
+  def tree_flatten(self) -> Tuple[Tuple[Any, ...], Hashable]:
     """Flattens this FrozenDict.
 
     Returns:
       A flattened version of this FrozenDict instance.
     """
-    return (self._dict,), ()
+    sorted_keys = sorted(self._dict)
+    return tuple([self._dict[k] for k in sorted_keys]), tuple(sorted_keys)
 
   @classmethod
-  def tree_unflatten(cls, _, data):
+  def tree_unflatten(cls, keys, values):
     # data is already deep copied due to tree map mechanism
     # we can skip the deep copy in the constructor
-    return cls(*data, __unsafe_skip_copy__=True)
+    return cls({k: v for k, v in zip(keys, values)}, __unsafe_skip_copy__=True)
+
+
+jax.tree_util.register_keypaths(
+    FrozenDict, lambda fd: tuple(jax.tree_util.DictKey(k) for k in sorted(fd))
+)
 
 
 def _prepare_freeze(xs: Any) -> Any:
@@ -189,7 +197,7 @@ def freeze(xs: Mapping[Any, Any]) -> FrozenDict[Any, Any]:
   return FrozenDict(xs)
 
 
-def unfreeze(x: FrozenDict[Any, Any]) -> Dict[Any, Any]:
+def unfreeze(x: Union[FrozenDict, Dict[str, Any]]) -> Dict[Any, Any]:
   """Unfreeze a FrozenDict.
 
   Makes a mutable copy of a `FrozenDict` mutable by transforming
@@ -205,7 +213,7 @@ def unfreeze(x: FrozenDict[Any, Any]) -> Dict[Any, Any]:
     # the dict branch would also work here but
     # it is much less performant because jax.tree_util.tree_map
     # uses an optimized C implementation.
-    return jax.tree_util.tree_map(lambda y: y, x._dict)  # pylint: disable=protected-access
+    return jax.tree_util.tree_map(lambda y: y, x._dict)  # type: ignore
   elif isinstance(x, dict):
     ys = {}
     for key, value in x.items():
@@ -215,9 +223,10 @@ def unfreeze(x: FrozenDict[Any, Any]) -> Dict[Any, Any]:
     return x
 
 
-def copy(x: Dict[str, Any], add_or_replace: Dict[str, Any]) -> Dict[str, Any]:
+def copy(x: Union[FrozenDict, Dict[str, Any]], add_or_replace: Union[FrozenDict, Dict[str, Any]]) -> Union[FrozenDict, Dict[str, Any]]:
   """Create a new dict with additional and/or replaced entries. This is a utility
-  function for regular dicts that mimics the behavior of `FrozenDict.copy`.
+  function that can act on either a FrozenDict or regular dict and mimics the
+  behavior of `FrozenDict.copy`.
 
   Example::
 
@@ -230,12 +239,16 @@ def copy(x: Dict[str, Any], add_or_replace: Dict[str, Any]) -> Dict[str, Any]:
     A new dict with the additional and/or replaced entries.
   """
 
-  new_dict = jax.tree_map(lambda x: x, x) # make a deep copy of dict x
-  new_dict.update(add_or_replace)
-  return new_dict
+  if isinstance(x, FrozenDict):
+    return x.copy(add_or_replace)
+  elif isinstance(x, dict):
+    new_dict = jax.tree_map(lambda x: x, x) # make a deep copy of dict x
+    new_dict.update(add_or_replace)
+    return new_dict
+  raise TypeError(f'Expected FrozenDict or dict, got {type(x)}')
 
 
-def pop(x: Dict[str, Any], key: str) -> Tuple[Dict[str, Any], Any]:
+def pop(x: Union[FrozenDict, Dict[str, Any]], key: str) -> Tuple[Union[FrozenDict, Dict[str, Any]], Any]:
   """Create a new dict where one entry is removed. This is a utility
   function for regular dicts that mimics the behavior of `FrozenDict.pop`.
 
@@ -250,9 +263,13 @@ def pop(x: Dict[str, Any], key: str) -> Tuple[Dict[str, Any], Any]:
     A pair with the new dict and the removed value.
   """
 
-  new_dict = jax.tree_map(lambda x: x, x) # make a deep copy of dict x
-  value = new_dict.pop(key)
-  return new_dict, value
+  if isinstance(x, FrozenDict):
+    return x.pop(key)
+  elif isinstance(x, dict):
+    new_dict = jax.tree_map(lambda x: x, x) # make a deep copy of dict x
+    value = new_dict.pop(key)
+    return new_dict, value
+  raise TypeError(f'Expected FrozenDict or dict, got {type(x)}')
 
 
 def _frozen_dict_state_dict(xs):
