@@ -23,13 +23,15 @@ from typing import (Any, Callable, Dict, Generic, Iterable, Mapping, Optional,
                     Sequence, Set, Tuple, TypeVar, Union)
 
 from flax.ids import uuid
-from flax import configurations as config
+from flax import config as config
+from flax import configurations as legacy_config  # only for flax_lazy_rng
 from flax import errors
 from flax import struct
 from flax import traceback_util
 from .frozen_dict import freeze
 from .frozen_dict import FrozenDict
 from .frozen_dict import unfreeze
+from . import partial_eval
 from . import tracers
 from . import meta
 import jax
@@ -95,7 +97,7 @@ class LazyRng(struct.PyTreeNode):
   @staticmethod
   def create(rng: Union['LazyRng', PRNGKey],
              *suffix: PRNGFoldable) -> 'LazyRng':
-    if not config.flax_lazy_rng:
+    if not legacy_config.flax_lazy_rng:
       if isinstance(rng, LazyRng):
         assert not rng.suffix
         rng = rng.rng
@@ -403,7 +405,7 @@ class Scope:
   <https://github.com/google/flax/tree/main/tests/core/design>`_
   for a number of examples using ``Scopes``.
   """
-  reservations: Set[str]
+  reservations: Dict[str, Optional[str]]
 
   def __init__(self,
                variables: MutableVariableDict,
@@ -437,7 +439,7 @@ class Scope:
     self.trace_level = tracers.trace_level(tracers.current_trace())
 
     self.rng_counters = {key: 0 for key in self.rngs}
-    self.reservations = set()
+    self.reservations = dict()
 
     self._invalid = False
 
@@ -485,17 +487,21 @@ class Scope:
     """Invalidates the Scope."""
     self._invalid = True
 
-  def mutable_variables(self) -> VariableDict:
+  def mutable_variables(self) -> Union[VariableDict, Dict[str, Any]]:
     """Returns an immutable copy of the mutable variables belonging to this Scope."""
     self._populate_collections()
     xs = {k: v for k, v in self._variables.items()
           if in_filter(self.mutable, k)}
-    return freeze(xs)
+    if config.flax_return_frozendict:
+      return freeze(xs)
+    return xs
 
-  def variables(self) -> VariableDict:
+  def variables(self) -> Union[VariableDict, Dict[str, Any]]:
     """Returns an immutable copy of the variables belonging to this Scope."""
     self._populate_collections()
-    return freeze(self._variables)
+    if config.flax_return_frozendict:
+      return freeze(self._variables)
+    return self._variables
 
   def _validate_trace_level(self):
     tracers.check_trace_level(self.trace_level)
@@ -517,20 +523,39 @@ class Scope:
       scope.rng_counters = self.rng_counters
     return scope
 
-  def reserve(self, name: str):
+  def name_reserved(self, name: str, col: Optional[str] = None) -> bool:
+    """Checks whether a name for a child Scope or Variable is taken.
+
+    Args:
+      name: the name to check for collision.
+      col: if a variable, the collection used.
+    """
+    if name in self.reservations:
+      # with relaxed naming, allow the same name for two variables in
+      # different collections, otherwise raise error.
+      if config.flax_relaxed_naming:
+        if (self.reservations[name] is None or col is None
+            or self.reservations[name] == col):
+            return True
+      else:
+        return True
+    return False
+
+  def reserve(self, name: str, col: Optional[str] = None):
     """Reserves a name for a child Scope or Variable.
 
     Throws an error if the name exists already.
 
     Args:
       name: the name to reserve.
+      col: if a variable, the collection used.
     """
     if not isinstance(name, str):
       raise TypeError('The type of scope "{name}" should be string but '
                       f'it is {type(name)}')
-    if name in self.reservations:
+    if self.name_reserved(name, col):
       raise ValueError(f'Duplicate use of scope name: "{name}"')
-    self.reservations.add(name)
+    self.reservations[name] = col
 
   def default_name(self, prefix: str) -> str:
     """Generates an unreserved name with the given prefix.
@@ -762,7 +787,7 @@ class Scope:
     Returns:
       The variable.  Throws an error if the variable exists already.
     """
-    self.reserve(name)
+    self.reserve(name, col)
     if not self.has_variable(col, name):
       if not self.is_mutable_collection(col) or init_fn is None:
         if self.is_collection_empty(col):
@@ -789,7 +814,7 @@ class Scope:
     Returns:
       The parameters. Throws an error if the params exist already.
     """
-    self.reserve(name)
+    self.reserve(name, 'params')
     if self.has_variable('params', name):
       abs_rng = jax.ShapeDtypeStruct(random.default_prng_impl().key_shape,
                                      jnp.uint32)
@@ -896,7 +921,7 @@ def apply(fn: Callable[..., Any],
   def wrapper(variables: VariableDict,
               *args,
               rngs: Optional[RNGSequences] = None,
-              **kwargs) -> Union[Any, Tuple[Any, VariableDict]]:
+              **kwargs) -> Union[Any, Tuple[Any, Union[VariableDict, Dict[str, Any]]]]:
     # Try to detect if user accidentally passed {'params': {'params': ...}.
     if 'params' in variables and isinstance(
         variables['params'],
@@ -943,6 +968,38 @@ def init(fn: Callable[..., Any],
   return wrapper
 
 
+def lazy_init(fn: Callable[..., Any],
+         mutable: CollectionFilter = True,
+         flags: Optional[Mapping] = None) -> Callable[..., Any]:
+  """Functionalizes a `Scope` function for lazy initialization.
+
+  Similair to ``init`` except that the init function now accepts
+  ``jax.ShapeDtypeStruct`` instances for arguments that do not
+  affect the variable initialization (typically this is all the input data).
+
+  Example::
+
+    def f(scope, x):
+        # the kernel init only uses the shape of x so we don't actually
+        # need a value for x and can pass it as a ShapeDtypeStruct in lazy_init.
+        k = scope.param("kernel", nn.initializers.lecun_normal(), (x.shape[-1], x.shape[-1]))
+        return x @ k
+    init_fn = lazy_init(f)
+    variables = init_fn(random.PRNGKey(0), jax.ShapeDtypeStruct((1, 128), jnp.float32))
+
+
+  Args:
+    fn: a function taking a `Scope` as its first argument.
+    mutable: the filter determining which variable collections are mutable.
+    flags: internal flags.
+
+  Returns:
+    `fn` with the scope partially applied. Unlike ``init`` which returns a tuple of function
+    output and variables, the lazy init function only returns the variables.
+  """
+  return partial_eval.lazy_init(lambda *args, **kwargs: init(fn, mutable, flags)(*args, **kwargs)[1])
+
+
 def _is_valid_collection(col: VariableDict):
   if not isinstance(col, (FrozenDict, dict)):
     return False
